#!/usr/bin/env python

"""
Directory repositories for WebStack.

Copyright (C) 2005 Paul Boddie <paul@boddie.org.uk>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
"""

import os

class DirectoryRepository:

    "A directory repository providing session-like access to files."

    def __init__(self, path, fsencoding=None, delay=1):

        """
        Initialise the repository using the given 'path' to indicate the
        location of the repository. If no such location exists in the filesystem
        an attempt will be made to create the directory.

        The optional 'fsencoding' parameter can be used to assert a particular
        character encoding used by the filesystem to represent filenames. By
        default, the default encoding is detected (or Unicode objects are used
        if appropriate).

        The optional 'delay' argument specifies the time in seconds between each
        poll of an opened repository file when that file is found to be locked
        for editing.
        """

        if not os.path.exists(path):
            os.mkdir(path)
        self.path = path
        self.fsencoding = fsencoding
        self.delay = delay

        # Guess the filesystem encoding.

        if fsencoding is None:
            if os.path.supports_unicode_filenames:
                self.fsencoding = None
            else:
                import locale
                self.fsencoding = locale.getdefaultlocale()[1]

        # Or override any guesses.

        else:
            self.fsencoding = fsencoding

    def _convert_name(self, name):
        if self.fsencoding:
            return name.encode(self.fsencoding)
        else:
            return name

    def _convert_fsname(self, name):
        if self.fsencoding:
            return unicode(name, self.fsencoding)
        else:
            return name

    def keys(self):
        # NOTE: Special names converted using a simple rule.
        l = []
        for name in os.listdir(self.path):
            if name.endswith(".edit"):
                l.append(name[:-5])
            else:
                l.append(name)
        return map(self._convert_fsname, l)

    def full_path(self, key):
        return os.path.join(self.path, self._convert_name(key))

    def has_key(self, key):
        return key in self.keys()

    # NOTE: Methods very similar to Helpers.Session.Wrapper.

    def items(self):
        results = []
        for key in self.keys():
            results.append((key, self[key]))
        return results

    def values(self):
        results = []
        for key in self.keys():
            results.append(self[key])
        return results

    def _wait_for_file(self, key):
        filename = self.full_path(key)
        if os.path.exists(filename) or os.path.exists(filename + ".edit"):
            while 1:
                try:
                    os.rename(filename, filename + ".edit")
                except OSError:
                    time.sleep(self.delay)
                else:
                    break
            return filename, 1
        else:
            return filename, 0

    def __delitem__(self, key):
        filename = self.full_path(key)
        if os.path.exists(filename) or os.path.exists(filename + ".edit"):
            while 1:
                try:
                    os.remove(filename)
                except OSError:
                    time.sleep(self.delay)
                else:
                    break
        else:
            raise KeyError, key

    def __getitem__(self, key):
        filename, exists = self._wait_for_file(key)
        if not exists:
            raise KeyError, key
        try:
            f = open(filename + ".edit", "rb")
            s = ""
            try:
                s = f.read()
            finally:
                f.close()
        finally:
            os.rename(filename + ".edit", filename)

        return s

    def __setitem__(self, key, value):
        filename, exists = self._wait_for_file(key)
        try:
            f = open(filename + ".edit", "wb")
            try:
                f.write(value)
            finally:
                f.close()
        finally:
            os.rename(filename + ".edit", filename)

# vim: tabstop=4 expandtab shiftwidth=4
